CREATE TABLE Cor (
  idCor INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  nomeCor VARCHAR(255) NULL,
  tipoCor ENUM('Sólida', 'Metálica', 'Perolizada') NULL,
  PRIMARY KEY(idCor)
);

CREATE TABLE Combustivel (
  idCombustivel INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  nomeCombustivel VARCHAR(255) NULL,
  PRIMARY KEY(idCombustivel)
);

CREATE TABLE Tipo (
  idTipo INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  nomeTipo VARCHAR(255) NULL,
  PRIMARY KEY(idTipo)
);

CREATE TABLE Marca (
  idMarca INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  nomeMarca VARCHAR(255) NULL,
  siteMarca VARCHAR(255) NULL,
  PRIMARY KEY(idMarca)
);

CREATE TABLE Cliente (
  idCliente INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  nomeCliente VARCHAR(255) NULL,
  email VARCHAR(255) NULL,
  cpf VARCHAR(255) NULL,
  endereco VARCHAR(255) NULL,
  telefone VARCHAR(255) NULL,
  senha VARCHAR(255) NOT NULL,
  admin TINYINT(1) NOT NULL,
  PRIMARY KEY(idCliente)
);

CREATE TABLE Acessorio (
  idAcessorio INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  nomeAcessorio VARCHAR(255) NULL,
  PRIMARY KEY(idAcessorio)
);

CREATE TABLE Modelo (
  idModelo INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  Marca_idMarca INTEGER UNSIGNED NOT NULL,
  Tipo_idTipo INTEGER UNSIGNED NOT NULL,
  nomeModelo VARCHAR(255) NULL,
  PRIMARY KEY(idModelo),
  INDEX Modelo_Tipo(Tipo_idTipo),
  INDEX Modelo_Marca(Marca_idMarca),
  FOREIGN KEY(Tipo_idTipo)
    REFERENCES Tipo(idTipo)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(Marca_idMarca)
    REFERENCES Marca(idMarca)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);

CREATE TABLE Automovel (
  idAutomovel INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  Combustivel_idCombustivel INTEGER UNSIGNED NOT NULL,
  Cliente_idCliente INTEGER UNSIGNED NOT NULL,
  Cor_idCor INTEGER UNSIGNED NOT NULL,
  Modelo_idModelo INTEGER UNSIGNED NOT NULL,
  anoFabricacao INTEGER UNSIGNED  NOT NULL,
  anoModelo INTEGER UNSIGNED NOT NULL,
  versao VARCHAR(255) NULL,
  motor VARCHAR(4) NULL,
  portas INTEGER UNSIGNED NULL,
  preco FLOAT NULL,
  quilometragem INTEGER UNSIGNED NULL,
  placa VARCHAR(7) NULL,
  descricao TEXT NULL,
  url_foto VARCHAR(255) NULL,
  PRIMARY KEY(idAutomovel),
  INDEX Automovel_FKIndex1(Modelo_idModelo),
  INDEX Automovel_FKIndex2(Cor_idCor),
  INDEX Automovel_FKIndex3(Cliente_idCliente),
  INDEX Automovel_FKIndex4(Combustivel_idCombustivel),
  FOREIGN KEY(Modelo_idModelo)
    REFERENCES Modelo(idModelo)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  FOREIGN KEY(Cor_idCor)
    REFERENCES Cor(idCor)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION,
  FOREIGN KEY(Cliente_idCliente)
    REFERENCES Cliente(idCliente)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  FOREIGN KEY(Combustivel_idCombustivel)
    REFERENCES Combustivel(idCombustivel)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);

CREATE TABLE Automovel_has_Acessorio (
  Automovel_idAutomovel INTEGER UNSIGNED NOT NULL,
  Acessorio_idAcessorio INTEGER UNSIGNED NOT NULL,
  PRIMARY KEY(Automovel_idAutomovel, Acessorio_idAcessorio),
  INDEX Automovel_has_Acessorio_FKIndex1(Automovel_idAutomovel),
  INDEX Automovel_has_Acessorio_FKIndex2(Acessorio_idAcessorio),
  FOREIGN KEY(Automovel_idAutomovel)
    REFERENCES Automovel(idAutomovel)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  FOREIGN KEY(Acessorio_idAcessorio)
    REFERENCES Acessorio(idAcessorio)
      ON DELETE NO ACTION
      ON UPDATE NO ACTION
);

CREATE UNIQUE INDEX indiceEmail ON Cliente(email);
   
CREATE TRIGGER deleta_automovel BEFORE DELETE ON Automovel
FOR EACH ROW 
DELETE FROM Automovel_has_Acessorio WHERE Automovel_has_Acessorio.Automovel_idAutomovel = OLD.idAutomovel; 

CREATE VIEW AutomovelCompleto as SELECT Automovel.*,Marca.nomeMarca,Marca.idMarca,Combustivel.nomeCombustivel,Combustivel.idCombustivel,Cliente.nomeCliente, Cliente.email, Cor.nomeCor, Cor.idCor,Modelo.nomeModelo,Modelo.idModelo,Tipo.nomeTipo,Tipo.idTipo FROM Automovel, Combustivel, Cliente, Cor, Modelo, Marca,Tipo WHERE Automovel.Combustivel_idCombustivel = Combustivel.idCombustivel AND Automovel.Cliente_idCliente = Cliente.idCliente AND Automovel.Cor_idCor = Cor.idCor AND Automovel.Modelo_idModelo = Modelo.idModelo AND Marca.idMarca = Modelo.Marca_idMarca;

CREATE VIEW tipo_ano_preco AS select idAutomovel,idTipo,nomeTipo,nomeModelo,anoFabricacao,anoModelo,preco,url_foto FROM Tipo, Modelo, Automovel WHERE Modelo.idModelo = Automovel.Modelo_idModelo AND Modelo.Tipo_idTipo = Tipo.idTipo; 

CREATE VIEW modelo_ano_preco AS select idAutomovel,nomeModelo,anoFabricacao,anoModelo,preco,url_foto FROM Automovel JOIN Modelo ON Automovel.Modelo_idModelo=Modelo.idModelo;

